import oracledb
from promptflow import tool
from promptflow.connections import CustomConnection


@tool
def run_query(connection: CustomConnection, query: str, params: dict[str, any]) -> list:
    # check custom connection for Oracle configuration
    assert connection.configs["user"] != None, "Oracle user is not set"
    assert connection.secrets["password"] != None, "Oracle password is not set"
    assert connection.configs["dsn"] != None, "Oracle dsn is not set"

    # establish connection
    con = oracledb.connect(user=connection.configs["user"],
                           password=connection.secrets["password"],
                           dsn=connection.configs["dsn"])

    # create cursor
    with con.cursor() as cur:
        # execute parameterized query
        # query would be SELECT * FROM table WHERE id = :id
        # params would be {'id': 1}
        print("Executing query: ", query)
        print("With params: ", params)
        cur.execute(query, **params)
        cols = [c[0] for c in cur.description]
        result = cur.fetchall()
        print("Intermediate Result")
        print(cols, result)

        items =[dict(zip(cols, row)) for row in result] 
        
    return items



import os
from promptflow.connections import CustomConnection
from oracle.tools.query import run_query

def test_run_single_query():
    con = CustomConnection(
        configs = {
            "user": "system",
            "dsn": "localhost/xe"
        },
        secrets = {
            "password": "abcd1234",
        }
    )
    result = run_query(con, query="SELECT * FROM SYS.CUSTOMERS WHERE CUSTOMER_ID = :id", 
                       params={ "id": 6 })
    
    assert len(result) == 1

def test_run_multiple_query():
    con = CustomConnection(
        configs ={
            "user": "system",
            "dsn": "localhost/xe"
        },
        secrets = {
            "password": "abcd1234",
        }
    )
    result = run_query(con, query="SELECT * FROM SYS.CUSTOMERS WHERE MEMBERSHIP = :membership", 
                       params={ "membership": "Base" })
    
    assert len(result) == 6


from setuptools import find_packages, setup

PACKAGE_NAME = "pforacle"

setup(
    name="pforacle",
    version="0.0.6",
    description="This package contains prompt flow tools for querying Oracle databases.",
    packages=find_packages(),
    install_requires=[
        'oracledb'
    ],
    entry_points={
        "package_tools": ["query = oracle.tools.utils:list_package_tools"],
    },
    include_package_data=True,   # This line tells setuptools to include files from MANIFEST.in
)